import { handleHighlightButtonPress } from './highlightButton';

const MINIMUM_SELECTION_LENGTH = 3;
const MAXIMUM_SELECTION_LENGTH = 2048;
const TOOLTIP_OFFSET = -60;

const tooltipHTML = `
	<div class="tooltip">
		<button class="tooltip_content" id="highlight-button">
			<div class="tooltip_indicator"></div>
			<div class="tooltip_text" id="example">
				<p id="another">Highlight</p>
			</div>
		</button>
	</div>
`;

const calculateBounds = (range, tooltip) => {
	// Get the bounding rectangle of the selection.
	const selectionBounds = range.getBoundingClientRect();
	tooltip.style.left = `${selectionBounds.left + (selectionBounds.width / 2)}px`;
	tooltip.style.top = `${selectionBounds.top + TOOLTIP_OFFSET}px`;
};

const initTooltip = () => {
	// Instantiate and populate the tooltip element, hidden by default.
	const tooltip = document.createElement('div');
		
	tooltip.innerHTML = tooltipHTML;
	tooltip.classList.add('tooltip', 'hidden');

	// Only target selection events on the article content body text. This prevents showing the tooltip
	// if the user selects anything else.
	const articleBody = document.querySelector('[data-attribute="article-content-body"]');

	articleBody.appendChild(tooltip);

	const highlightButton = document.getElementById('highlight-button');

	// Add an event listener to catch mouseup events. We cannot capture selection events because these
	// are emitted every the selection changes and we only want to capture the selection once the user
	// has finished selecting.
	articleBody.addEventListener('mouseup', () => {
		// Get the user's current selection.
		const selection = document.getSelection();
		const range = selection.getRangeAt(0);

		const text = selection && selection.toString();

		// Get all the child nodes of the article body.
		const nodes = Array.from(articleBody.childNodes);

		// Attempt to find a single paragraph within which the user's selection starts and ends. Users should
		// not be able to add annotations which span multiple paragraphs.
		const paragraph = nodes.find((n) => (n.nodeName.toLowerCase() === 'p' && n.contains(selection.anchorNode) && n.contains(selection.focusNode)));

		// Create an array from the selection's child nodes.
		const selectionNodes = Array.from(range.cloneContents().childNodes);

		// Create an array of the selection's node names (e.g. DIV, MARK etc.).
		const nodeNames = selectionNodes.map((n) => n.nodeName);

		// check if the selected text in a <mark> node  
		const containedInOtherMarks = selection?.focusNode?.parentNode?.nodeName === 'MARK'? true : false;

		// Check whether the selection overlaps an existing annotation.
		const overlap = nodeNames.includes('MARK');
		
		// Only show the tooltip if the selection:
		// * Is longer than the minimum length.
		// * Is shorter than the maximum length.
		// * Is in an article paragraph.
		// * Does not span multiple paragraphs.
		// * Does not overlap an existing annotation.
		if (
			text
			&& text.length > MINIMUM_SELECTION_LENGTH
			&& text.length < MAXIMUM_SELECTION_LENGTH
			&& paragraph
			&& !overlap
			&& !containedInOtherMarks
		) {
			// Make the tooltip visible.
			tooltip.classList.remove('hidden');
			// Position the tooltip where the user's selection is.
			calculateBounds(range, tooltip);
			
			window.addEventListener('scroll', () => {
				calculateBounds(range, tooltip);
			});

			// Add an onclick handler for the highlight button.
			highlightButton.onclick = () => handleHighlightButtonPress(selection, paragraph, text, tooltip);

			// If the user's selection is empty, hide the tooltip.
		} else {
			tooltip.classList.add('hidden');
		}		
	});
};

export { initTooltip };
